/*
 * (c) 2006-2008 Thomas Maier-Komor
 * LICENSE: GPLv2, see file COPYING for details
 */

#include <assert.h>
#include <curses.h>
#include <errno.h>
#include <fcntl.h>
#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <strings.h>
#include <syslog.h>
#include <sys/wait.h>
#include <sys/utsname.h>
#include <unistd.h>
#ifdef __sun
#include <ucontext.h>
#include <dlfcn.h>
#endif

#include "log.h"
#include "sysstat.h"

char **ARGV;
int ARGC;
void (*DfltSegV)(int signr) = 0;
static int Syslog = 0, Verbose = 0;

#ifdef DEBUG
static int Log = -1;


void dbug(const char *msg, ...)
{
	va_list val;
	char buf[8192];

	if (Log == -1)
		return;
	va_start(val,msg);
	vsnprintf(buf,sizeof(buf),msg,val);
	va_end(val);
	if (Syslog) {
		syslog(LOG_DEBUG,buf);
	}
	(void) write(Log,buf,strlen(buf));
}
#endif


void info(const char *msg, ...)
{
	va_list val;
	char buf[8192];

	if (Verbose == 0)
		return;
	va_start(val,msg);
	(void) vsnprintf(buf,sizeof(buf),msg,val);
	va_end(val);
	if (Syslog)
		syslog(LOG_INFO,buf);
#ifdef DEBUG
	else if (Log != -1)
		(void) write(Log,buf,strlen(buf));
#endif
	else
		(void) write(STDERR_FILENO,buf,strlen(buf));
}


void warn(const char *msg, ...)
{
	va_list val;
	char buf[8192] = "warning: ";

	va_start(val,msg);
	(void) vsnprintf(buf+9,sizeof(buf)-9,msg,val);
	va_end(val);
	if (Syslog)
		syslog(LOG_WARNING,buf+9);
#ifdef DEBUG
	else if (Log != -1)
		(void) write(Log,buf,strlen(buf));
#endif
	else
		(void) write(STDERR_FILENO,buf,strlen(buf));
}


void error(const char *msg, ...)
{
	va_list val;
	char buf[8192] = "error: ";

	va_start(val,msg);
	(void) vsnprintf(buf+7,sizeof(buf)-7,msg,val);
	va_end(val);
	if (Syslog)
		syslog(LOG_ERR,buf+7);
#ifdef DEBUG
	else if (Log != -1)
		(void) write(Log,buf,strlen(buf));
#endif
	else
		(void) write(STDERR_FILENO,buf,strlen(buf));
	close(STDERR_FILENO);
	exit(EXIT_FAILURE);
}


void log2syslog(const char *exename)
{
	openlog(exename,LOG_USER,0);
	Syslog = 1;
}


void setVerbose(const char *logfile)
{
#ifdef DEBUG
	Log = open(logfile,O_WRONLY|O_APPEND|O_CREAT|O_TRUNC,0666);
	if (Log == -1)
		error("could not create %s: %s\n",logfile,strerror(errno));
#else
	(void)logfile;
#endif
	Verbose = 1;
}


#ifdef DEBUG
void setLog(int l)
{
	Log = l;
}
#endif


void SegVhandler(int signr)
{
	struct utsname utsname;
	char msg[16<<10], *m = msg;
	char cmd[32];
	int tty = open("/dev/tty",O_RDWR), i = 0, s = 0;
	FILE *ps;

	if (tty == -1)
		tty = STDERR_FILENO;
	uname(&utsname);
	m += snprintf(msg,sizeof(msg),
		"\rSegV - please send the following information to software@maier-komor.de:\n\r"
		"sysstat version " VERSION "\n\r"
		"sysname: %s\n\r"
		"release: %s\n\r"
		"version: %s\n\r"
		"machine: %s\n\r"
		, utsname.sysname
		, utsname.release
		, utsname.version
		, utsname.machine
		);
	while (i < ARGC) {
		m += sprintf(m,"argv[%d] : %s\n\r",i,ARGV[i]);
		++i;
	}
	m += sprintf(m, "SegV occured at:\n\r");
#ifdef __sun
	seteuid(0);
	setuid(0);
	sprintf(cmd,"/usr/bin/pstack %d",getpid());
	ps = popen(cmd,"r");
	while (!feof(ps)) {
		char buf[1024];
		fgets(buf,sizeof(buf),ps);
		if (s) {
			strcat(m,buf);
			strcat(m,"\r");
		} else if (strstr(buf,"SegVhandler"))
			s = 1;
	}
	pclose(ps);
#else
	m += sprintf(m, "stacktrace unavailable - please analyze core file\n\r");
#endif
#ifdef DEBUG
	if (Log != STDERR_FILENO)
		(void) write(Log,msg,strlen(msg));
#endif
	(void) write(tty,msg,strlen(msg));
	if (DfltSegV)
		DfltSegV(signr);
	abort();
}


